#!/usr/bin/env python

# Introduction to Python RenderMan
# The first program I always write for a new (interactive) 3D system
# is a spinning cube. It's the graphical equivalent of "Hello World".
# Since this is RenderMan, I've upgraded to a spinning teapot.

# This Python program uses the Ri class, which is basically the
# RenderMan RIB file format with Python syntax. The program code
# is thus a 1:1 translation of teapot.rib, although divided into
# functions to work with my other Python support programs.
# (And the color is different.)

import os
from os import path

import prman

def drawTeapot():
    """ Draw teapot as single element within larger scene """
    # The teapot will draw by itself, but I always like to have a
    # a minimum set of attributes
    Ri.AttributeBegin()
    Ri.Bxdf("PxrDiffuse", "smooth", {"color diffuseColor":  [0, 1, 1]})
    Ri.Geometry("teapot")
    Ri.AttributeEnd()

def render(**args):
    """ Draw teapot as the entire scene, during development with rmwatch.py
        This function should not call Begin, Display, Format
        """
    frame = args.get("frame", 1)
    # Camera setup
    Ri.Projection(Ri.PERSPECTIVE, {Ri.FOV: 90})
    Ri.Translate(0, -2, 8)
    Ri.Rotate(-15, 1, 0, 0)
    #ERRROR!!
    # Scene
    Ri.WorldBegin()
    Ri.Rotate(frame * 2, 0, 1, 0)
    Ri.Rotate(-90, 1, 0, 0)
    drawTeapot()
    Ri.WorldEnd()

##  If run stand-alone, set up output context then render()

if __name__ == "__main__":
    # The Python API declares Ri as a class, but I prefer to use
    # old style C coding with prefixed names.
    # This only works for a single file!
    globals()["Ri"] = prman.Ri()
    fName = path.splitext(path.basename(__file__))[0] + ".tiff"
    Ri.Begin(Ri.RENDER)
    Ri.Display(fName, "framebuffer", "rgb", { "string format": "uint8"})
    # This only works for "rgb" display formats. If the display is
    # "rgba" background pixels not drawn by any object will be
    # left transparent, even though the previewer shows this color.
    Ri.DisplayFilter("PxrBackgroundDisplayFilter", "background", {
                        "color backgroundColor": [ 0, 0, 0 ] })
    Ri.Format(1280, 720, 1)
    Ri.Integrator("PxrDefault", "integrator")
    Ri.FrameBegin(1)
    render(frame=1, filename=fName)
    Ri.FrameEnd()
    Ri.End()

